<?php
/*
	Plugin Name: BBQ Pro - Log Requests
	Plugin URI: https://plugin-planet.com/bbq-pro-log-requests/
	Description: Logs bad requests blocked by BBQ Pro.
	Author: Jeff Starr
	Author URI: https://monzillamedia.com/
	Donate link: https://monzillamedia.com/donate.html
	Requires at least: 5.1
	Tested up to: 6.1
	Version: 2.2
*/

// == DO NOT EDIT NEXT LINE ==

if (!defined('ABSPATH')) exit; // security bro. never change.





// == EDIT THESE VARIABLES ==

// Log blocked requests

define('BBQ_PRO_LOG_ENABLE', false); // true = enable logging, false = disable logging

define('BBQ_PRO_LOG_FILENAME', 'bbq-pro-log.txt'); // specify name of log file (must be writable and protected)


// Send email alerts

define('BBQ_PRO_ALERTS_ENABLE', true); // true = enable email alerts, false = disable email alerts

define('BBQ_PRO_ALERTS_NAME', ''); // name for email alerts

define('BBQ_PRO_ALERTS_EMAIL', ''); // email address for alerts

define('BBQ_PRO_ALERTS_SUBJECT', ''); // subject for alerts





// == DO NOT EDIT BELOW THIS LINE ==

function bbq_log_requests($match, $request_uri, $query_string, $user_agent, $referrer, $protocol, $ip_address, $the_request) {
	
	if (!isset($match[0]) || empty($match[0])) return;
	
	$match = isset($match[0]) ? $match[0] : 'undefined';
	
	$blog_name = BBQ_PRO_ALERTS_NAME  ? BBQ_PRO_ALERTS_NAME  : get_bloginfo('name');
	
	$admin_email = BBQ_PRO_ALERTS_EMAIL ? BBQ_PRO_ALERTS_EMAIL : get_bloginfo('admin_email');
	
	$subject = BBQ_PRO_ALERTS_SUBJECT ? BBQ_PRO_ALERTS_SUBJECT : 'BBQ Alert: Blocked Request at '. $blog_name;
	
	$log = bbq_log_requests_data(array($match, $request_uri, $query_string, $user_agent, $referrer, $protocol, $ip_address, $the_request));
	
	$data  = 'Request Details:'. "\n\n" . $log;
	
	$data = stripslashes(trim($data));
	
	$headers  = 'X-Mailer: BBQ Pro'. "\n";
	$headers .= 'From: '. $blog_name .' <'. $admin_email .'>'. "\n";
	$headers .= 'Reply-To: '. $blog_name .' <'. $admin_email .'>'. "\n";
	$headers .= 'Content-Type: text/plain; charset="'. get_option('blog_charset') .'"'. "\n";
	
	if (BBQ_PRO_LOG_ENABLE) bbq_log_requests_file($log);
		
	if (BBQ_PRO_ALERTS_ENABLE) wp_mail($admin_email, $subject, $data, $headers);
	
}
add_action('bbq_scan', 'bbq_log_requests', 10, 8);

function bbq_log_requests_file($log) {
	
	$date = date('Y-m-d @ H:i:s', current_time('timestamp'));
	
	$data = '~=== '. $date .' ===~'. "\n" . "\n" . $log . "\n" .'~===~'. "\n";
	
	$file = plugin_dir_path(__FILE__) . BBQ_PRO_LOG_FILENAME;
	
	if (file_exists($file)) {
		
		$fopen = fopen($file, 'a');
		
	} else {
		
		$fopen = fopen($file, 'w');
		
	}
	
	fwrite($fopen, $data);
	
	fclose($fopen);	
	
}

function bbq_log_requests_data($data) {
	
	list ($match, $request_uri, $query_string, $user_agent, $referrer, $protocol, $ip_address, $the_request) = $data;
	
	$n = "\n";
	
	$data  = 'Pattern Match: '. $match . $n;
	$data .= 'Request URI: '  . $request_uri . $n;
	$data .= 'The Request: '  . $the_request . $n;
	$data .= 'Query String: ' . $query_string . $n;
	$data .= 'Referrer: '     . $referrer . $n;
	$data .= 'Protocol: '     . $protocol . $n;
	$data .= 'IP Address: '   . $ip_address . $n;
	$data .= 'User Agent: '   . $user_agent . $n;
	
	return $data;
	
}